#include "io430.h"
#include "radio.c"
#pragma diag_suppress=Pa082
#pragma diag_suppress=Pe111

void Clock_config();				// function prototypes
void Ports_config();
void SPI_config();
void Timer0_config();
void WDT_config();
void delay();
void Send_Radio(const char*, char);
void Send_Strobe(char);
void CC1101_config();

const char packet[] = {0x7F, 6, 0xAA, 0xBB, 'M', 'A', 'I', 'L'}; // radio packet
const char* dataPtr;				// pointer to byte to send via SPI
char dataCnt;						// # of remaining bytes to send

#define COMPAR  	P6				// comparator output at P2.6
#define ACC_PWR 	P7				// enable accelerometer on P2.7
#define CS	    	P2				// radio CS on P1.2
#define GDO0    	P3				// radio GDO0 signal at P1.3
#define GDO2    	P4				// radio GDO2 signal at P1.4
#define SCLK		P5				// SPI interface
#define MOSI		P6				// -- " --
#define MISO		P7				// -- " --

int main()
{
	WDTCTL = WDTPW + WDTHOLD;		// disable Watchdog
	Clock_config();					// set 8 MHz MSCK, 2 MHz SMCLK
	Timer0_config();				// set up timer for short delays	
	Ports_config();					// set up ports for GPIO
	SPI_config();					// configure USI module for SPI
	CC1101_config();				// init CC1101 radio
	WDT_config();					// configure WDT for 0.75 sec periodic wake-ups

	while(1)						// main app loop
	{  
		__low_power_mode_3();		// wake-up by WDT every ~0.75 sec	
		P2OUT_bit.ACC_PWR = 1;		// power-up Accelerometer and do power-up delay
		delay();					// comparator needs about 4ms to setup its output   
		if (!P2IN_bit.COMPAR) 		// transmit package?
		{ 
		  	Send_Radio(packet, 8);	// load packet to FIFO
			Send_Strobe(STX);		// enter TX mode
			while(!P1IN_bit.GDO0){}	// wait for sending the SYNC word
			while(P1IN_bit.GDO0){}	// wait for the end of TX
			Send_Strobe(SPWD);		// enter power down mode
		}
		P2OUT_bit.ACC_PWR = 0;		// power down Accelerometer
	}
	return 0;
}

void Send_Strobe(char s)			// send strobe s to the radio
{
	Send_Radio(&s, 1);  
}

void Send_Radio(const char* a, char l)// send array a of bytes via SPI; l is its length
{
  	dataCnt = l;
	dataPtr = a;
  	P1OUT_bit.CS = 0;				// CS down
	while (P1IN_bit.GDO2){}			// wait for radio interface ready
	USICTL1_bit.USIIE = 1;			// enable USI counter interrupt 
	__low_power_mode_0();			// Enter LPM0, interrupts enabled
	P1OUT_bit.CS = 1;				// CS up
}  

void CC1101_config()
{ 
	char i;
	for (i=0; i<10; i++) delay();	// radio power-up delay
  	dataCnt = 48;					// send that many 
	dataPtr = cc1101_regs;			//    config bytes
	P1OUT_bit.CS = 0;				// CS down
	while (P1IN_bit.MISO){}			// wait for radio interface ready
	USICTL1_bit.USIIE = 1;			// enable USI counter interrupt 
	__low_power_mode_0();			// Enter LPM0, interrupts enabled
	P1OUT_bit.CS = 1;				// CS up 
	Send_Radio(OUT_MODE, 3);		// set +10dBm power, enter SHDN mode
}

void WDT_config()
{
	WDTCTL = WDTPW + WDTTMSEL + WDTCNTCL + WDTSSEL
	  + WDTIS0;						// clock WDT from ACLK, 1:8192 divisor
	IFG1 = 0;						// clear WDT interrupt flag
	IE1 = WDTIE;					// enable WDT interrupt
}

void delay()
{
  	TA0CTL_bit.TACLR = 1;			// clear timer counter
  	TA0CTL_bit.MC0 = 1;				// start timer in Up mode
	__low_power_mode_3();			// wait for timer to expire
	TA0CTL_bit.MC0 = 0;				// stop timer
}

void Timer0_config()				// Timer0 setup for short delays
{
  	TA0CTL = TASSEL0 + TACLR + TAIE;// ACLK, enable interrupt
	TA0CCR0 = 120;					// setup for ~10ms delay	
}

void Ports_config()
{
  	P1OUT = CS + MISO;				// CS up
	P1DIR = CS + P0 + P1 + MOSI + SCLK;	// output on those pins
	P1REN = MISO;					// enable pull-up resistors
	P1SEL = SCLK + MISO + MOSI;		// assign SPI pins to USI
	P1SEL2 = 0;						// init PORT2
	P2OUT = 0;
	P2DIR = ACC_PWR;				// output on this pin
	P2SEL = 0;						// I/O function on P2.6 and P2.7
}

void Clock_config()					// configure basic clock module
{
  	DCOCTL = CALDCO_8MHZ;			// set 8 MHz clock
	BCSCTL1 = CALBC1_8MHZ;
	BCSCTL2 = DIVS1;				// SMCLK=DCO/4
	BCSCTL3 = LFXT1S1;				// select VLO as ACLK  
}

void SPI_config()					// configure USI module for SPI
{
	USICTL0 = USIPE7 + USIPE6 + USIPE5 + USIMST + USIGE + USIOE;
	USICKCTL = USISSEL1;			// use 1:1 SMCLK divider
}  

#pragma vector=USI_VECTOR			// USI SPI ISR routine
__interrupt void USI_ISR(void)
{
  	if (dataCnt)
	{
	 	USISRL = *dataPtr;			// load data to send to SPI shift reg
		USICNT = 8;					// transmit 8 bits 
		dataPtr++;
		dataCnt--;
	}  
	else
	{  
		USICTL1_bit.USIIE = 0;		// disable USI interrupt   
		__low_power_mode_off_on_exit();
	}	
}

#pragma vector=TIMER0_A1_VECTOR		// Timer0 ISR routine
__interrupt void Timer0_ISR(void)
{
	TA0CTL_bit.TAIFG = 0;			// clear TimerA interrupt
	__low_power_mode_off_on_exit();
}  

#pragma vector=WDT_VECTOR			// WDT ISR routine
__interrupt void WDT_ISR(void)
{
 	IFG1 = 0;						// clear WDT interrupt flag
	__low_power_mode_off_on_exit();
}